@@ -91,6 +91,41 @@ _Static_assert(_Alignof(pkt_t) <= _Alignof(uint16_t), "No padding for pkt");
 struct conn;
 typedef struct conn conn_t;
 
+// I keep running into various warnings or analyzer issues with the various
+// ways I've cheated with cheap FP-conversion methods to square root a size_t
+// and output an unsigned.  So we'll just actually provide a proper integer
+// square root method to move past all this.  It only gets used once to size a
+// buffer on thread start, so perf really isn't critical.
+//
+// This code is more or less copypasta from a stackexchange answer and then
+// cleaned up for style and such, but I verified it produces the right answers
+// for all the allowed range of values we use it for.
+static unsigned sqrt_sizet(size_t input)
+{
+    // This is basically the inverse of CLZ, how many value bits to the right
+    // of the leading zeros?
+    size_t shiftme = input;
+    size_t valbits = 0;
+    while (shiftme) {
+        valbits++;
+        shiftme >>= 1;
+    }
+
+    // Approx starting point
+    size_t answer = 1LLU << (valbits >> 1);
+    if (valbits & 1)
+        answer += answer >> 1;
+
+    // Trial divisions to home in
+    size_t tmp;
+    do {
+        tmp = input / answer;
+        answer = (answer + tmp) >> 1;
+    } while (answer > tmp);
+
+    return answer;
+}
+
 // per-thread state
 typedef struct {
     // These pointers and values are fixed for the life of the thread:
@@ -398,7 +433,7 @@ static bool conn_write_packet(thread_t* thr, conn_t* conn, size_t resp_size)
 {
     gdnsd_assert(resp_size);
     tcp_pkt_t* tpkt = thr->tpkt;
-    tpkt->pktbuf_size_hdr = htons((uint16_t)resp_size);
+    tpkt->pktbuf_size_hdr = htons((uint16_t)resp_size); // cppcheck-suppress redundantInitialization
     const size_t resp_send_size = resp_size + 2U;
     const ev_io* readw = &conn->read_watcher;
     const ssize_t send_rv = send(readw->fd, tpkt->pktbuf_raw, resp_send_size, 0);
@@ -1045,7 +1080,7 @@ void* dnsio_tcp_start(void* thread_asvoid)
     // Set up the conn_t churn buffer, which saves some per-new-connection
     // memory allocation churn by saving up to sqrt(max_clients) old conn_t
     // storage for reuse
-    thr.churn_alloc = lrint(floor(sqrt(addrconf->tcp_clients_per_thread)));
+    thr.churn_alloc = sqrt_sizet(thr.max_clients);
     gdnsd_assert(thr.churn_alloc >= 4U); // because tcp_cpt min is 16U
     thr.churn = xmalloc_n(thr.churn_alloc, sizeof(*thr.churn));
 
