@@ -115,7 +115,7 @@ typedef union {
 } pkt_t;
 
 F_HOT F_NONNULLX(1, 2, 3)
-unsigned process_dns_query(dnsp_ctx_t* ctx, const gdnsd_anysin_t* sa, pkt_t* packet, dso_state_t* dso, const unsigned packet_len);
+unsigned process_dns_query(dnsp_ctx_t* ctx, const gdnsd_anysin_t* sa, pkt_t* pkt, dso_state_t* dso, const unsigned packet_len);
 
 F_NONNULL F_WUNUSED F_RETNN
 dnsp_ctx_t* dnspacket_ctx_init_udp(dnspacket_stats_t** stats_out, const bool is_ipv6);
