@@ -34,6 +34,6 @@ F_NONNULL F_MALLOC
 char* statio_serialize(size_t* dlen_p);
 
 F_NONNULL
-void statio_deserialize(uint64_t* data, size_t dlen);
+void statio_deserialize(const uint64_t* data, size_t dlen);
 
 #endif // GDSND_STATIO_H
